#include "bspline.h"
#include <malloc.h>
#include <stdio.h>

struct Sbspline {
  GLfloat *ptsControle; /* vecteur de points de controle rangés x,y,z,... */
  int n; /* n+1 points de controle */
  int k; /* ordre */
  int m; /* degre */
  float *vecNodal; /* Vecteur Nodal */
  float def[2]; /* Intervalle de définition */
  
  GLfloat *ptsCourbe;  /* Points de la courbe pour l'affichage */
  int nbPtsCourbe;  /* Nombre de point de la courbe à afficher */
  
  float *ptsFloraison; /* Points de controle d'entrée de la floraison */
  
  GLuint *indexPolyControl;  /* Tableau d'index pour le tracé des points de contrôle de la courbe */
  GLuint *indexCourbe; /* Tableau d'index pour le tracé des points de la courbe */
};



void vecteurNodalUniforme (Bspline b) {
  
  int i;
  for (i=0;i<b->n+1+b->k;i++) b->vecNodal[i] = (float)i;
  
  b->def[0] = b->vecNodal[b->k-1];
  b->def[1] = b->vecNodal[b->n+1];
}

void vecteurNodalOuvertUniforme (Bspline b) {
  
  int i;
  int k = b->k;
  int nbPtsControle = b->n+1;
  
  for (i=0;i<k;i++) b->vecNodal[i] = 0.;
  for (i=k;i<nbPtsControle+1;i++) b->vecNodal[i] = (float)(i-k+1);
  for (i=nbPtsControle+1;i<nbPtsControle+k;i++) b->vecNodal[i] = (float)(nbPtsControle-k+1);
  
  b->def[0] = b->vecNodal[b->k-1];
  b->def[1] = b->vecNodal[b->n+1];
}

void initValeursBspline (Bspline b, GLfloat *ptsControle, int nbPtsControle, int k) {
  int i;
  
  b->k = k;
  b->m = k-1;
  b->n = nbPtsControle-1;
  
  b->ptsControle = (GLfloat*)malloc(sizeof(GLfloat)*nbPtsControle*3);
  for(i=0;i<nbPtsControle*3;i++) b->ptsControle[i] = ptsControle[i]; 
  
  b->ptsFloraison = (float*)malloc(sizeof(float)*k*3);
  
  b->vecNodal = (float*)malloc(sizeof(float)*(nbPtsControle+k));
  
  b->indexPolyControl = (GLuint*)malloc(sizeof(GLuint)*nbPtsControle);
  for(i=0;i<nbPtsControle;i++) b->indexPolyControl[i] = (GLuint)i; 
}

Bspline creerBsplineUniforme (GLfloat *ptsControle, int nbPtsControle, int k) {
  Bspline b = (Bspline)malloc(sizeof(struct Sbspline));
  
  initValeursBspline (b, ptsControle, nbPtsControle, k);
  vecteurNodalUniforme (b);
  
  return b;
}

Bspline creerBsplineOuvertUniforme (GLfloat *ptsControle, int nbPtsControle, int k) {
  Bspline b = (Bspline)malloc(sizeof(struct Sbspline));
  
  initValeursBspline (b, ptsControle, nbPtsControle, k);
  vecteurNodalOuvertUniforme (b);
  
  return b;
}

void calculCourbeBspline (Bspline b, int nbPtsCourbe) {
  int i;
  float u;
  float pas = (b->def[1] - b->def[0]) / ((float)nbPtsCourbe-1.);
  float eps = pas*0.01;
  
  b->ptsCourbe = (GLfloat*)malloc(sizeof(GLfloat)*nbPtsCourbe*3);
  b->indexCourbe = (GLuint*)malloc(sizeof(GLuint)*nbPtsCourbe);
  for (i=0;i<nbPtsCourbe;i++) b->indexCourbe[i] = (GLuint)i;
  
  b->nbPtsCourbe = nbPtsCourbe;
  
  i=0;
  for (u=b->def[0];u<=b->def[1]-eps;u+=pas) {
    evalBspline (b,u,&b->ptsCourbe[i*3]);
    i++;
  } 
  evalBspline (b,b->def[1],&b->ptsCourbe[i*3]);
}

void affichePolyControleBspline (Bspline b){
	
	glEnableClientState (GL_VERTEX_ARRAY);
	
	glVertexPointer (3,GL_FLOAT,0,b->ptsControle);
	
	glLineWidth (1.);
	
	glDrawElements (GL_LINE_STRIP,b->n+1,GL_UNSIGNED_INT,b->indexPolyControl);
	
	glDisableClientState (GL_VERTEX_ARRAY);
}

void afficheCourbeBspline (Bspline b) {
        int i;
	
	glEnableClientState (GL_VERTEX_ARRAY);
	
	glVertexPointer (3,GL_FLOAT,0,b->ptsCourbe);
	
	glLineWidth (3.);
	
	glDrawElements (GL_LINE_STRIP,b->nbPtsCourbe,GL_UNSIGNED_INT,b->indexCourbe);
	
	glLineWidth (1.);
	
	glDisableClientState (GL_VERTEX_ARRAY);
  
}


void evalRecBspline (Bspline b, float u, Point p, int dec, int k) {

 /* Code de la fonction récursive de la floraison */
 
 /* A Ecrire !!!!!! */
 
}


void evalBspline (Bspline b, float u, Point p) {

 /* Initialisation de la floraison et appel de la fonction récursive evalRecBspline */ 
 
 /* A Ecrire !!!!!! */
  
}



void intervalleDefinitionBspline (Bspline b, float def[2]){
  def[0] = b->def[0];
  def[1] = b->def[1];
}
